"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.gameClock = void 0;
var System_1 = require("./System");
var events_1 = require("./events");
function scheduleRaf(callback) {
    var id = requestAnimationFrame(callback);
    return function () { return cancelAnimationFrame(id); };
}
function scheduleTimeout(callback) {
    var id = setTimeout(callback, 16);
    return function () { return clearTimeout(id); };
}
var scheduleDefault = typeof requestAnimationFrame === 'function' ? scheduleRaf : scheduleTimeout;
function gameClock(ticksPerSecond, schedule) {
    if (ticksPerSecond === void 0) { ticksPerSecond = 60; }
    if (schedule === void 0) { schedule = scheduleDefault; }
    var deltaMs = 1000 / ticksPerSecond;
    return System_1.system(events_1.InitEvent, function (world) {
        var lastTime = Date.now();
        update();
        function update() {
            world.run(function () {
                var now = Date.now();
                while (lastTime < now) {
                    world.emit(new events_1.UpdateTick(now, 1 / ticksPerSecond));
                    lastTime += deltaMs;
                }
                world.emit(new events_1.RenderTick());
            });
            schedule(update);
        }
    });
}
exports.gameClock = gameClock;
//# sourceMappingURL=gameClock.js.map