var Entity = /** @class */ (function () {
    function Entity(onChange) {
        var _this = this;
        this.components = {};
        this.didNotify = false;
        this.notify = function () {
            if (!_this.didNotify) {
                _this.didNotify = true;
                onChange(_this);
            }
        };
    }
    Entity.prototype.add = function (component, value) {
        if (this.has(component)) {
            throw new Error('Component type already present.');
        }
        this.components[component] = value;
        this.notify();
        return this;
    };
    Entity.prototype.has = function (component) {
        return Object.prototype.hasOwnProperty.call(this.components, component);
    };
    Entity.prototype.get = function (component) {
        var value = this.components[component];
        if (!value) {
            throw new TypeError('Component type not present.');
        }
        return value;
    };
    Entity.prototype.remove = function (component) {
        delete this.components[component];
        this.notify();
        return this;
    };
    return Entity;
}());
export { Entity };
//# sourceMappingURL=Entity.js.map